"""
Config file for Rougail-structural_commandline

Silique (https://www.silique.fr)
Copyright (C) 2024

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


def get_rougail_config(
    *,
    backward_compatibility=True,
) -> dict:
    options = """
structural_commandline:
  description: Configuration rougail-structural_commandline
  commandline: false
  add_extra_options:
    description: Add extra options to tiramisu-cmdline-parser
    default: true
"""
    return {
        "name": "exporter",
        "process": "structural",
        "options": options,
        "level": 20,
    }


__all__ = "get_rougail_config"
